/*******************************************************************
 *
 *	File:		FrameTicker.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32base.h>
#include <e32std.h>
#include <e32svr.h>
#include "FrameTicker.h"


CFrameTicker::CFrameTicker(TInt aFps) : CActive(EPriorityHigh), iFps(aFps)
{
}


CFrameTicker::~CFrameTicker()
{
	Cancel();
	iTimer.Close();
}


CFrameTicker* CFrameTicker::NewL(TInt aFps)
{
	CFrameTicker* zoomDrawer = new(ELeave)CFrameTicker(aFps);
	CleanupStack::PushL(zoomDrawer);
	zoomDrawer->ConstructL();
	CleanupStack::Pop();
	return 	zoomDrawer;
}

void CFrameTicker::ConstructL()
{
	User::LeaveIfError(iTimer.CreateLocal());
	CActiveScheduler::Add(this);
}

void CFrameTicker::AddFrameTickObserver(MFrameTickObserver* aFrameTickObserver, MFrameTickObserver* aFrameTickerObservers[])
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!aFrameTickerObservers[i])
		{
			aFrameTickerObservers[i] = aFrameTickObserver;
			break;
		}
	}
}

void CFrameTicker::RemoveFrameTickObserver(MFrameTickObserver* aFrameTickObserver, MFrameTickObserver* aFrameTickerObservers[])
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (aFrameTickerObservers[i] == aFrameTickObserver)
			aFrameTickerObservers[i] = NULL;
	}
}

void CFrameTicker::RemoveFrameTickObserver(MFrameTickObserver* aFrameTickObserver)
{
	RemoveFrameTickObserver(aFrameTickObserver, iBeforeFrameTickObservers);
	RemoveFrameTickObserver(aFrameTickObserver, iAfterFrameTickObservers);
	RemoveFrameTickObserver(aFrameTickObserver, iDrawFrameTickObservers);
}

void CFrameTicker::AddFrameTickObserver(MFrameTickObserver* aFrameTickObserver, TInt aAtWhatTime)
{
	if (aAtWhatTime == EBeforeDrawFrame)
		AddFrameTickObserver(aFrameTickObserver, iBeforeFrameTickObservers);
	else if (aAtWhatTime == EAfterDrawFrame)
		AddFrameTickObserver(aFrameTickObserver, iAfterFrameTickObservers);
	else if (aAtWhatTime == EDrawFrame)
		AddFrameTickObserver(aFrameTickObserver, iDrawFrameTickObservers);
}

void CFrameTicker::Start(MFrameTickObserver* aFrameTickObserver, TInt aAtWhatTime)
{
	AddFrameTickObserver(aFrameTickObserver, aAtWhatTime);

	if (iClientCount == 0)
	{
		iFrameDelay = (1000 * 1000) / iFps;
		iTimer.After(iStatus, iFrameDelay);
		SetActive();
	}
	iClientCount++;
}

void CFrameTicker::Stop(MFrameTickObserver* aFrameTickObserver)
{
	RemoveFrameTickObserver(aFrameTickObserver);

	iClientCount--;
	ASSERT(iClientCount>=0);

	if (iClientCount == 0)
		Cancel();
}



void CFrameTicker::RunL()
{
	iTimer.After(iStatus, iFrameDelay);
	SetActive();

//	RDebug::Print(_L("EZOOM: TICK"));

	TInt i;
	for (i=0 ; i<KMaxObservers ; i++)
		if (iBeforeFrameTickObservers[i])
			iBeforeFrameTickObservers[i]->FrameTick();
	for (i=0 ; i<KMaxObservers ; i++)
		if (iDrawFrameTickObservers[i])
			iDrawFrameTickObservers[i]->FrameTick();
	for (i=0 ; i<KMaxObservers ; i++)
		if (iAfterFrameTickObservers[i])
			iAfterFrameTickObservers[i]->FrameTick();
}


void CFrameTicker::DoCancel()
{
	iTimer.Cancel();
}

