/*******************************************************************
 *
 *	File:		Hook.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32base.h>
#include <e32std.h>
#include <w32adll.h>
#include <bitdraw.h>

#include "ControllerBase.h"
class CHookedFbsDrawDevice : public CFbsDrawDevice
{
public:
	~CHookedFbsDrawDevice()
	{
		if (iOrig)
			User::Panic(_L("ZOOMANIM"), KErrNotSupported);
	}

	virtual TDisplayMode DisplayMode() const
	{
		return iOrig->DisplayMode();
	}

	virtual TInt LongWidth() const
	{
		return iOrig->LongWidth();
	}

	virtual void MapColors(const TRect& aRect,const TRgb* aColors,TInt aNumPairs,TBool aMapForwards)
	{
		iOrig->MapColors(aRect, aColors, aNumPairs, aMapForwards);
	}

	virtual void ReadLine(TInt aX,TInt aY,TInt aLength,TAny* aBuffer,TDisplayMode aDispMode) const
	{
		iOrig->ReadLine(aX, aY, aLength, aBuffer, aDispMode);
	}

	virtual TRgb ReadPixel(TInt aX,TInt aY) const
	{
		return iOrig->ReadPixel(aX, aY);
	}

	virtual TUint32* ScanLineBuffer() const
	{
		return iOrig->ScanLineBuffer();
	}

	virtual TInt ScanLineBytes() const
	{
		return iOrig->ScanLineBytes();
	}

	virtual TDisplayMode ScanLineDisplayMode() const
	{
		return iOrig->ScanLineDisplayMode();
	}

	virtual TSize SizeInPixels() const
	{
		return iOrig->SizeInPixels();
	}

	virtual TInt HorzTwipsPerThousandPixels() const
	{
		return iOrig->HorzTwipsPerThousandPixels();
	}

	virtual TInt VertTwipsPerThousandPixels() const
	{
		return iOrig->VertTwipsPerThousandPixels();
	}

	virtual void OrientationsAvailable(TBool aOrientation[4])
	{
		iOrig->OrientationsAvailable(aOrientation);
	}

	virtual TBool SetOrientation(TOrientation aOrientation)
	{
		return iOrig->SetOrientation(aOrientation);
	}

	virtual void WriteBinary(TInt aX,TInt aY,TUint32* aBuffer,TInt aLength,TInt aHeight,TRgb aColor,CGraphicsContext::TDrawMode aDrawMode)
	{
		iOrig->WriteBinary(aX, aY, aBuffer, aLength, aHeight, aColor, aDrawMode);
	}

	virtual void WriteBinaryLine(TInt aX,TInt aY,TUint32* aBuffer,TInt aLength,TRgb aColor,CGraphicsContext::TDrawMode aDrawMode)
	{
		iOrig->WriteBinaryLine(aX, aY, aBuffer, aLength, aColor, aDrawMode);
	}

	virtual void WriteBinaryLineVertical(TInt aX,TInt aY,TUint32* aBuffer,TInt aHeight,TRgb aColor,CGraphicsContext::TDrawMode aDrawMode,TBool aUp)
	{
		iOrig->WriteBinaryLineVertical(aX, aY, aBuffer, aHeight, aColor, aDrawMode, aUp);
	}

	virtual void WriteRgb(TInt aX,TInt aY,TRgb aColor,CGraphicsContext::TDrawMode aDrawMode)
	{
		iOrig->WriteRgb(aX, aY, aColor, aDrawMode);
	}

	virtual void WriteRgbMulti(TInt aX,TInt aY,TInt aLength,TInt aHeight,TRgb aColor,CGraphicsContext::TDrawMode aDrawMode)
	{
		iOrig->WriteRgbMulti(aX, aY, aLength, aHeight, aColor, aDrawMode);
	}

	virtual void WriteRgbAlphaLine(TInt aX,TInt aY,TInt aLength,TUint8* aRgbBuffer,TUint8* aMaskBuffer)
	{
		iOrig->WriteRgbAlphaLine(aX, aY, aLength, aRgbBuffer, aMaskBuffer);
	}

	virtual void WriteLine(TInt aX,TInt aY,TInt aLength,TUint32* aBuffer,CGraphicsContext::TDrawMode aDrawMode)
	{
		iOrig->WriteLine(aX, aY, aLength, aBuffer, aDrawMode);
	}

public:
	virtual TInt InitScreen()
	{
		return iOrig->InitScreen();
	}

	virtual void SetAutoUpdate(TBool aAutoUpdate)
	{
		iOrig->SetAutoUpdate(aAutoUpdate);
	}

	virtual void SetBits(TAny* aBits)
	{
		iOrig->SetBits(aBits);
	}

	virtual TInt SetCustomPalette(const CPalette* aPalette)
	{
		return iOrig->SetCustomPalette(aPalette);
	}

	virtual TInt GetCustomPalette(CPalette*& aPalette) 
	{
		return iOrig->GetCustomPalette(aPalette);
	}

	virtual void SetDisplayMode(CFbsDrawDevice* aFbsDrawDevice)
	{
		iOrig->SetDisplayMode(aFbsDrawDevice);
	}

	virtual void SetDitherOrigin(const TPoint& aPoint)
	{
		iOrig->SetDitherOrigin(aPoint);
	}

	virtual void SetUserDisplayMode(TDisplayMode aDisplayMode)
	{
		iOrig->SetUserDisplayMode(aDisplayMode);
	}

	virtual void SetShadowMode(TShadowMode aShadowMode)
	{
		iOrig->SetShadowMode(aShadowMode);
	}

	virtual void SetFadingParameters(TUint8 aBlackMap,TUint8 aWhiteMap)
	{
		iOrig->SetFadingParameters(aBlackMap, aWhiteMap);
	}

	virtual void ShadowArea(const TRect& aRect)
	{
		iOrig->ShadowArea(aRect);
	}

	virtual void ShadowBuffer(TInt aParam, TUint32* aParam1)
	{
		iOrig->ShadowBuffer(aParam, aParam1);
	}

	virtual void Update()
	{
		iOrig->Update();
//		iController->Redraw();
	}

	virtual void Update(const TRegion& aRegion)
	{
		iOrig->Update(aRegion);
		iController->Redraw();
	}

	virtual void UpdateRegion(const TRect& aRect)
	{
		iOrig->UpdateRegion(aRect);
		iController->Redraw(aRect);
	}

public:
	virtual void WriteRgbAlphaMulti(TInt aX,TInt aY,TInt aLength,TRgb aColor,const TUint8* aMaskBuffer)
	{
		iOrig->WriteRgbAlphaMulti(aX, aY, aLength, aColor, aMaskBuffer);
	}

	virtual void Reserved_2()
	{
		iOrig->Reserved_2();
	}

	virtual void Reserved_3()
	{
		iOrig->Reserved_3();
	}

	virtual void Reserved_4()
	{
		iOrig->Reserved_4();
	}

	virtual void Reserved_5()
	{
		iOrig->Reserved_5();
	}

	CFbsDrawDevice*		iOrig;
	CControllerBase*	iController;
};


// get rid of some warnings
#pragma warning(disable: 4510 4610)
class CMyFbsScreenDevice : public CFbsScreenDevice
{
public:
	CFbsDrawDevice* DrawDevice()
	{
		return iDrawDevice;
	}
	void SetDrawDevice(CFbsDrawDevice* aFbsDrawDevice)
	{
		iDrawDevice = aFbsDrawDevice;
	}
};
#pragma warning(default: 4510 4610)

