/*******************************************************************
 *
 *	File:		ReadImage.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "ReadImage.h"


CReadImage::CReadImage(const TSize& aSize, TBool aKeepRatio, TDisplayMode aDisplayMode, MReadImageObserver& aReadImageObserver)
	: iSize(aSize), iKeepRatio(aKeepRatio), iDisplayMode(aDisplayMode), iReadImageObserver(aReadImageObserver)
{
}

CReadImage::~CReadImage()
{
	delete iConvertUtil;
	delete iScaler;
	delete iBitmap;
}

CReadImage* CReadImage::NewL(const TSize& aSize, TBool aKeepRatio, TDisplayMode aDisplayMode, MReadImageObserver& aReadImageObserver)
{
	CReadImage*	self = new(ELeave)CReadImage(aSize, aKeepRatio, aDisplayMode, aReadImageObserver);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

void CReadImage::ConstructL()
{
	iConvertUtil = CMdaImageFileToBitmapUtility::NewL(*this);
}


void CReadImage::ReadImage(const TDesC& aFilename)
{
	TRAPD(err, iConvertUtil->OpenL(aFilename));
	if (err)
		iReadImageObserver.ReadImageComplete(err);
}


void CReadImage::ConvertL()
{
	TFrameInfo	frameInfo;
	iConvertUtil->FrameInfo(0, frameInfo);
	
	iBitmap = new (ELeave) CFbsBitmap();
	User::LeaveIfError(iBitmap->Create(frameInfo.iFrameCoordsInPixels.Size(), iDisplayMode));
	iConvertUtil->ConvertL(*iBitmap);
}


void CReadImage::MiuoCreateComplete(TInt /*aError*/)
{
}

void CReadImage::MiuoOpenComplete(TInt aError)
{
	if (aError == KErrNone)
	{
		TRAP(aError, ConvertL());
	}
	if (aError)
		iReadImageObserver.ReadImageComplete(aError);
}

void CReadImage::ScaleL()
{
	iScaler = CMdaBitmapScaler::NewL();
	iScaler->ScaleL(*this, *iBitmap, iSize, iKeepRatio);
}
	
void CReadImage::MiuoConvertComplete(TInt aError)
{
		// called twice. First when ConvertL() completes and later for ScaleL()
	if (aError == KErrNone)
	{
		if (!iScaler)
		{
			TRAP(aError, ScaleL());
			if (aError == KErrNone)
				return;
		}
	}
	iReadImageObserver.ReadImageComplete(aError);
}



