/*******************************************************************
 *
 *	File:		ZoomAnim.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32base.h>
#include <e32std.h>
#include <w32adll.h>

#include "FrameBufferMgr.h"
#include "FrameTicker.h"
#include "ZoomAnim_If.h"
#include "ZoomController.h"
#include "InputManager.h"
#include "hook.h"


/*********************************************************
 *
 * class CZoomAnim
 *
 *********************************************************/


const TInt KFramesPerSecond = 15;

class CZoomAnim : public CWindowAnim
{
public:
	~CZoomAnim();
	CZoomAnim();

public:		// from CAnim
	virtual TInt CommandReplyL(TInt aOpcode, TAny* aArgs);
	virtual void Command(TInt /*aOpcode*/, TAny* /*aArgs*/){}
	virtual void Animate(TDateTime* /*aDateTime*/){}
	virtual TBool OfferRawEvent(const TRawEvent& aRawEvent);

public:
	virtual void ConstructL(TAny* aArgs, TBool aHasFocus);
	virtual void Redraw(){}
	virtual void FocusChanged(TBool /*aState*/){}
protected:
	void Destruct();

	void StartZooming();
	void StopZooming();

	void UnhookScreenRedraws();
	void HookScreenRedraws();

	CFrameBufferMgrBase*	iFrameBufferMgr;
	CFrameTicker*			iFrameTicker;
	CZoomController*		iController;
	TInputManager			iInputManager;

	CHookedFbsDrawDevice	iHookedFbsDrawDevice;
	TBool					iStarted;
};


CZoomAnim::~CZoomAnim()
{
	Destruct();
}

CZoomAnim::CZoomAnim()
{
}


void CZoomAnim::ConstructL(TAny* /*aArgs*/, TBool /*aHasFocus*/)
{
	iFrameTicker = CFrameTicker::NewL(KFramesPerSecond);
	iInputManager.SetFrameTicker(iFrameTicker);

	iFrameBufferMgr = CreateFrameBufferMgrL(iFrameTicker);

	iController = CZoomController::NewL(*iFrameBufferMgr, *iFrameTicker, *iFunctions->ScreenDevice());

	iInputManager.SetController(iController);
}


void CZoomAnim::Destruct()
{
	StopZooming();

	delete iController;
	iController = NULL;

	delete iFrameBufferMgr;
	iFrameBufferMgr = NULL;

	delete iFrameTicker;
	iFrameTicker = NULL;
}


void CZoomAnim::HookScreenRedraws()
{
	CMyFbsScreenDevice*		myFbsScreenDevice = (CMyFbsScreenDevice*)iFunctions->ScreenDevice();
	CFbsDrawDevice*			fbsDrawDevice = myFbsScreenDevice->DrawDevice();
	
	iHookedFbsDrawDevice.iOrig = fbsDrawDevice;
	iHookedFbsDrawDevice.iController = iController;
	myFbsScreenDevice->SetDrawDevice(&iHookedFbsDrawDevice);
	iHookedFbsDrawDevice.Update();
}


void CZoomAnim::UnhookScreenRedraws()
{
	CMyFbsScreenDevice*		myFbsScreenDevice = (CMyFbsScreenDevice*)iFunctions->ScreenDevice();
	
	myFbsScreenDevice->SetDrawDevice(iHookedFbsDrawDevice.iOrig);
	iHookedFbsDrawDevice.iOrig = NULL;
	iHookedFbsDrawDevice.iController = NULL;
}


void CZoomAnim::StartZooming()
{
	if (!iStarted)
	{
		iFrameBufferMgr->RemapFrameBuffer();

		HookScreenRedraws();

		iController->Start();

		iFunctions->GetRawEvents(ETrue);

		iStarted = ETrue;
	}
}

void CZoomAnim::StopZooming()
{
	if (iStarted)
	{
		UnhookScreenRedraws();

		iController->Stop();

		iFrameBufferMgr->RestoreFrameBuffer();

		iFunctions->GetRawEvents(EFalse);

		iStarted = EFalse;
	}
}

TInt CZoomAnim::CommandReplyL(TInt aOpcode, TAny* aArgs)
{
	switch (aOpcode)
	{
		case ZoomAnimCmdStart:
			StartZooming();
			break;

		case ZoomAnimCmdStop:
			StopZooming();
			break;

		case ZoomAnimCmdConfigure:
		{
			const TZoomAnimConfig* zoomAnimConfig = (const TZoomAnimConfig*) aArgs;
			iController->SetTransparency(zoomAnimConfig->iTransparency);
			iController->SetBgImage(zoomAnimConfig->iBgImageFileName);
			break;
		}

		default:
			break;

	}
	return KErrNone;
}


TBool CZoomAnim::OfferRawEvent(const TRawEvent& aRawEvent)
{
	return iInputManager.OfferRawEvent(aRawEvent);
}


/*********************************************************
 *
 * class CZoomAnimDll
 *
 *********************************************************/

class CZoomAnimDll : public CAnimDll
{
public:
	virtual CAnim *CreateInstanceL(TInt /*aType*/);
};


CAnim* CZoomAnimDll::CreateInstanceL(TInt /*aType*/)
{
	return new(ELeave)CZoomAnim;
}


EXPORT_C CAnimDll* CreateCAnimDllL()
{
	return new(ELeave) CZoomAnimDll();
}

TInt E32Dll(TDllReason)
{
	return(KErrNone);
}

