/*******************************************************************
 *
 *	File:		ZoomBase.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __ZOOM_BASE_H
#define __ZOOM_BASE_H

enum TRotation
{
	ERot0,
	ERot90,
	ERot180,
	ERot270
};


class CBitmap : CBase
{
public:
	~CBitmap();
	static CBitmap* NewL(const TSize& aSize);
	const TUint16* Bitmap() const {return iBitmap;}
	TUint16* Bitmap() {return iBitmap;}
	TSize Size() const {return iSize;}

	void CopyBitmapData(const CFbsBitmap& aFbsbitmap, const TRect& aSrc, const TPoint& aDst);
protected:
	CBitmap(const TSize& aSize) : iSize(aSize){}
	void ConstructL();
	TUint16*	iBitmap;
	TSize		iSize;
};

const TInt KButtonWidth = 16;
const TInt KButtonHeight = 16;

enum TCommand
{
	ECmdNone = -1,
	ECmdQuality = 0,
	ECmdMode,
	ECmdRotate,
	ECmdClose,
	KNoOfButtons
};

enum
{
	ESizeSmall,
	ESizeMedium,
	ESizeLarge
};

class CTextControl : CBase
{
public:
	~CTextControl();
	static CTextControl* NewL(const TSize& aSize, TDisplayMode aDisplayMode);
	CBitmap*	Bitmap() {return iRawBitmap;}
	void SetTextL(const TDesC& aText);
	TBool HasText() const {return iHasText;}
	void SetHasText(TBool aHasText) {iHasText = aHasText;}

protected:
	CTextControl(TDisplayMode aDisplayMode) : iDisplayMode(aDisplayMode){}
	void ConstructL(const TSize& aSize);

	TDisplayMode		iDisplayMode;
	CFbsBitmap*			iBitmap;
	CFbsBitmapDevice*	iBitmapDevice;
	CFbsBitGc*			iGc;
	CFont*				iFont;

	CBitmap*			iRawBitmap;
	TBool				iHasText;
};


class CZoomControl : public CBase
{
public:
	~CZoomControl();
	static CZoomControl* NewL(const TSize& aScreenSize, TDisplayMode aDisplayMode);

	void LoadButtonBitmap(TInt aSlot, TInt aId, TBool aPressed = EFalse);
	void SetBackgroundImage(const CFbsBitmap& aBitmap);
	void ClearBackground();

	TBool	IsPointInZoomControl(const TPoint& aPoint) const;
	TBool	IsPointInToolBar(const TPoint& aPoint) const;
	TBool	IsPointInScreenRect(const TPoint& aPoint) const;

	TBool	IsButtonPressed(const TPoint& aPoint) const;
	TBool	SetButtonPressed(TInt aSlot, TBool aPressed);

	TRect	MoveControl(const TPoint& aDelta);
	void	SetViewPort(const TRect& aRect);
	void	SetVisibleRect(const TRect& aRect){iVisibleRect = aRect;}

		// rects in screen coordinates
	TRect		ControlRectOnScreen() const {return iControlRect;}
	TRect		ToolBarRectOnScreen() const {return ToScreenCoord(iToolBarRect);}
	TRect		ScaledUpScreenRectOnScreen() const {return ToScreenCoord(iScaledUpScreenRect);}
	TRect		ScaledDownScreenRectOnScreen() const {return ToScreenCoord(iScaledDownScreenRect);}
	TRect		StatusBarRectOnScreen() const {return ToScreenCoord(iStatusbarRect);}
	TRect		ButtonRectOnScreen(TInt aSlot) const {return ToScreenCoord(iButtons[aSlot]);}

	CBitmap*		Bitmap() {return iBitmap;}
	CTextControl*	StatusbarControl() {return iStatusbar;}

protected:
	CZoomControl(const TSize& aScreenSize, TDisplayMode aDisplayMode);
	void ConstructL();
	void FillRect(const TRect& aRect, TRgb aColor);
	void FillRect(const TRect& aRect, TUint16 aColor);
	void LoadButtonBitmapL(TInt aSlot, TInt aId, TBool aPressed);
	void CopyBitmapData(const CFbsBitmap* aFbsbitmap, CBitmap* aBitmap, TInt aSlot, TBool aPressed);
	TRect	ToScreenCoord(const TRect& aRect) const;

	TSize			iScreenSize;
	CBitmap*		iBitmap;
	CTextControl*	iStatusbar;

		// rects related to the CBitmap
	TRect			iControlRect;			// rect of the control
	TRect			iToolBarRect;			// rect in iControlRect for the toolbar
	TRect			iScaledUpScreenRect;	// rect in iControlRect representing the scaled up screen
	TRect			iScaledDownScreenRect;	// rect in iControlRect representing the screen (sub rect of iScaledUpScreenRect)
	TRect			iStatusbarRect;			// rect in iControlRect representing status text

	TRect			iVisibleRect;



	TRgb			iToolbarColor;
	TRect			iButtons[KNoOfButtons];
	TInt			iButtonsBitmapId[KNoOfButtons];
	TDisplayMode	iDisplayMode;
	TInt			iSlotButtonPressed;
};



TUint16* PixelAddress(TUint16* aBitmap, const TPoint& aPoint, TInt aWidth);
const TUint16* PixelAddress(const TUint16* aBitmap, const TPoint& aPoint, TInt aWidth);
void ConfineRect(TRect& aRect, const TRect& aBoundingRect);
void SplitRects(const TRect& aRect, const TRect& aBoundingRect, TRect& aRect1, TRect& aRect2, TRect& aRect3, TRect& aRect4, TRect& aRect5);

#endif			/* __ZOOM_BASE_H */
