/*******************************************************************
 *
 *	File:		ZoomController.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __ZOOM_CONTROLLER_H
#define __ZOOM_CONTROLLER_H

#include "ControllerBase.h"
#include "ZoomBase.h"
#include "ZoomModel.h"
#include "ReadImage.h"
#include "FrameTicker.h"


class CFrameBufferMgrBase;
class CFrameTicker;
class TZoomModel;
class CZoomDrawer;

/************************************************
 *
 * Zoom controller
 *
 ************************************************/

class CZoomController : public CControllerBase, public MReadImageObserver, public MFrameTickObserver
{
public:
	~CZoomController();
	static CZoomController* NewL(CFrameBufferMgrBase& aFrameBufferMgr, CFrameTicker& aFrameTicker, const CFbsScreenDevice& aFbsScreenDevice);
	void SetTransparency(TInt aTransparency);
	void SetBgImage(const TDesC& aileName);

public:		// implements CZoomControllerBase
	virtual TBool ModifyRawEvent(TRawEvent& aRawEvent);
	virtual TBool HandleKeyEvent(TInt aScanCode, TBool& aWantRepeat);

	virtual TBool HandlePointerDown(const TPoint& aPoint);
	virtual TBool HandlePointerUp(const TPoint& aPoint);
	virtual TBool HandlePointerMove(const TPoint& aPoint);

	virtual void Redraw(const TRect& aRect);
	virtual void Redraw();

	virtual void Start();
	virtual void Stop();

public:			// implements MFrameTickObserver
	virtual void FrameTick();

public:			// implements MReadImageObserver
	virtual void ReadImageComplete(TInt aError);
	
protected:
	TBool KeepingTicking();
	void StartTicker();
	void StopTicker();
	TBool	iIsTickerStarted;
	TBool	iRunning;
	TBool	iTickContinuously;

protected:
	void LoadButtonBitmaps();
	void LoadButtonBitmapL(TInt aId);
	void CopyBitmapData(const CFbsBitmap* aCFbsbitmap, CBitmap* aBitmap, TInt aId);

protected:
	CZoomController(CFrameBufferMgrBase& aFrameBufferMgr, CFrameTicker& aFrameTicker, const CFbsScreenDevice& aFbsScreenDevice);
	void ConstructL();

	void DrawFrame();
	void IncreaseZoom(TInt aDelta);
	void MoveViewPort(const TPoint& aDelta);
	void CycleRotation();
	void SetToolActive(TBool aActive);
	void SetQuality(TBool aHigh);
	void SetMode(TBool aAutoMode);

	void HandleButtonPressed(TInt aSlotId);

	void RedrawScreen();

	void NotifyApp();

	void Flash();
	void StopFlashing();
	void StartFlashing();
	void SetStatusbarTextL(const TDesC& aText);
	TBool					iIsFlashing;
	TInt					iFlashFrame;
	TInt					iFlashStep;
	TBool					iFlashOn;
	TInt					iFlashNextFlashFrame;
	TInt					iNoOfFLashSteps;
	TBuf<128>				iText;

	TBool					iCloseRequested;

	TBool					iIsDragging;
	TBool					iIsDraggingViewPort;
	TInt					iButtonSlot;
	TPoint					iPoint;

	TBool					iZoomControlActive;

	TRect					iRedrawRect;
	TZoomModel				iModel;
	CZoomDrawer*			iDrawer;
	CFrameBufferMgrBase&	iFrameBufferMgr;
	CFrameTicker&			iFrameTicker;
	const CFbsScreenDevice&	iFbsScreenDevice;

	CZoomControl*			iZoomControl;

protected:
	void ReadImageL(const TDesC& aFileName);
	CReadImage*				iReadImage;
};


#endif			/* __ZOOM_CONTROLLER_H */
