/*******************************************************************
 *
 *	File:		ZoomDrawer.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __ZOOM_DRAWER_H
#define __ZOOM_DRAWER_H

#include "ZoomBase.h"
class TZoomModel;


class CZoomDrawer : public CBase
{
public:
	~CZoomDrawer(){}
	CZoomDrawer(TUint16* aFb, TUint16* aShadowFb, TZoomModel& aScaledFullScreenModel, CZoomControl& aZoomControl, TDisplayMode aDisplayMode);
	void	SetZoomControlActive(TBool aActive){iZoomControlActive=aActive;}

	void DrawFrame();
	void DrawFrame(const TRect& aRect);
	void QualityChanged();

	void ClearRect(const TRect& aRect);
	void ClearBorders();

	void SetTransparency(TInt aTransparency){iAlpha = (TUint8)aTransparency;}

protected:
	TUint16 InterpolatePixelRgb565(TUint16 aPixel0, TUint16 aPixel1, TInt aScale);
	TUint16 InterpolatePixelRgb444(TUint16 aPixel0, TUint16 aPixel1, TInt aScale);

	TUint16 AlphaBlendRgb565(TUint16 aPixel0, TUint16 aPixel1, TUint8 aAlpha);
	TUint16 AlphaBlendRgb444(TUint16 aPixel0, TUint16 aPixel1, TUint8 aAlpha);

		// bilinear scaling routines
	void DrawFrame_BL_RGB565(const TRect& aRect);
	void DrawFrame_BL_RGB444(const TRect& aRect);
	void DrawFrame_BL_RGB565_T(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint);
	void DrawFrame_BL_RGB444_T(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint);
	void DrawFrame_BL_RGB565_A(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint, TBool aExtraAlpha);
	void DrawFrame_BL_RGB444_A(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint, TBool aExtraAlpha);

		// next-neighbour scaling routines
	void DrawFrame_NN_RGB565(const TRect& aRect);
	void DrawFrame_NN_RGB444(const TRect& aRect);
	void DrawFrame_NN_RGB565_T(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint);
	void DrawFrame_NN_RGB444_T(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint);
	void DrawFrame_NN_RGB565_A(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint, TBool aExtraAlpha);
	void DrawFrame_NN_RGB444_A(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint, TBool aExtraAlpha);

	void BitBlt(const TRect& aDst, const CBitmap& aBitmap, const TPoint& aPoint);

	void DrawAlphaBlendedZoomControl(const TRect& aRect);
	void DrawTransparentToolbar(const TRect& aRect);
	void DrawZoomControl(const TRect& aRect);
protected:
	TDisplayMode	iDisplayMode;
	TBool			iZoomControlActive;

	TUint8			iAlpha;
	TUint16			iFixedColor;
	TUint16			iTransparentColor;
	TUint16			iToolbarColor;

protected:
	typedef void (CZoomDrawer::*TFrameDrawer)(const TRect& aRect);
	TFrameDrawer		iFrameDrawer;

	typedef void (CZoomDrawer::*TFrameAlphaDrawer)(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint, TBool aExtraAlpha);
	TFrameAlphaDrawer	iFrameAlphaDrawer;

	typedef void (CZoomDrawer::*TFrameTransparentDrawer)(const TRect& aRect, const CBitmap* aBitmap, const TPoint& aPoint);
	TFrameTransparentDrawer	iFrameTransparentDrawer;

protected:
	TUint16*		iFb;
	TUint16*		iShadowFb;
	TZoomModel&		iModel;

	CZoomControl&	iZoomControl;
};


#endif			/* __ZOOM_DRAWER_H */
