/*******************************************************************
 *
 *	File:		ZoomModel.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __ZOOM_MODEL_H
#define __ZOOM_MODEL_H

#include "ZoomBase.h"

class TZoomModel
{
public:
	TZoomModel();
	void Construct(const TSize& aSize);

		// model operations
	void	SetZoomFactor(TInt aPercent);
	TBool	IncreaseZoom(TInt aPercent);
	TBool	MoveViewPort(const TPoint& aDelta);
	void	CycleRotation();
	void	SetQuality(TBool aHigh);

		// calculations
	TPoint	ScaleUp(const TPoint& aPoint) const;
	TPoint	ScaleDown(const TPoint& aPoint) const;
	void	ScaleDownPrecise(const TPoint& aSrcPoint, TPoint& aDstPoint, TInt& aDx, TInt& aDy) const;
	TRect	ScaleUp(const TRect& aRect) const;
	TPoint	ScaleDownAndRotateToViewPort(const TPoint& aPoint) const;
	TPoint	RotateBack(const TPoint& aSrc) const;
	TPoint	Rotate(const TPoint& aSrc) const;
	TPoint	MoveToViewPort(const TPoint& aSrc) const;
	TRect	VisibleArea() const;

	TRect	ScaleViewPort(const TSize& aScaledScreenSize) const;
	TPoint	ScalePoint(const TPoint& aPoint, const TSize& aScaledScreenSize) const;
	TPoint	ScalePointUp(const TPoint& aPoint, const TSize& aScaledScreenSize) const;

		// basic attributes
	TBool			iHighQuality;		// ETrue --> bilinear scaling, EFalse --> next neighnour
	TInt			iZoomPercent;
	TRotation		iRotation;
	TInt			iDelta;				// derived from  iZoomPercent

		// used by drawer; based on iRotation figure out in which directions to redraw
	TInt			iDirectionX;
	TInt			iDirectionY;

		// used by drawer to figure out where to start outputting pixels
	TInt			iDstPointHeight;

		// sizes
	TSize			iScreenSize;		// 208 x 320 for P800
	TSize			iScaledScreenSize;	// iScreenSize * iZoomPercent
	TRect			iViewPort;			// sub rect in TRect(TPoint(0, 0), iScaledScreenSize);

		// fixed-point scaling
	TInt			iScaleTab[101];
	
};


#endif			/* __ZOOM_MODEL_H */
