/*******************************************************************
 *
 *	File:		Dialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Dialogs.h"
#include <f32file.h>
#include <QikSelectMediaFileDlg.h>
#include <eikchlst.h>
#include <QikSlider.h>
#include <qiknumbereditor.h>
#include <eikedwin.h>

#include <txtrich.h>				// CRichText
#include <eikrted.h>				// CEikRichTextEditor

#include "CSimpleTextParser.h"
#include "ezoom.hrh"
#include <ezoom.rsg>



/*********************************************
 *
 * CGeneralOptionsDialog
 *
 *********************************************/

enum
{
	EBgImageOptionNone = 0,
	EBgImageOptionUser,
	EBgImageOptionSelect
};


CGeneralOptionsDialog::CGeneralOptionsDialog(TZoomOptions& aZoomOptions)
	: iZoomOptions(aZoomOptions)
{
}
	

TBool CGeneralOptionsDialog::OkToExitL(TInt /*aButtonId*/)
{
	if (iBgImageChoiceList->CurrentItem() == 0)
		iZoomOptions.iBgImageFilename = _L("");

	iZoomOptions.iTransparency = iTransparencySlider->CurrentValue();

	return ETrue;
}


void CGeneralOptionsDialog::PreLayoutDynInitL()
{
	iBgImageChoiceList	= STATIC_CAST(CEikChoiceList*, Control(ECtlOptionsBgImage));
	if (iZoomOptions.iBgImageFilename.Length() > 0)
		iBgImageChoiceList->SetCurrentItem(1);

	iTransparencySlider = STATIC_CAST(CQikSlider*, Control(ECtlOptionsTransparency));
	iTransparencySlider->SetValue(iZoomOptions.iTransparency);

	SetUserDefinedL();
}

void CGeneralOptionsDialog::SetUserDefinedL()
{
	TParsePtr		parse(iZoomOptions.iBgImageFilename);
	
	if (iZoomOptions.iBgImageFilename.Length() > 0)
	{
		iBgImageChoiceList->DesCArray()->Delete(1);
		iBgImageChoiceList->DesCArray()->InsertL(1, parse.NameAndExt());
	}
}

void CGeneralOptionsDialog::PostLayoutDynInitL()
{
}



void CGeneralOptionsDialog::HandleControlStateChangeL(TInt aControlId)
{
	if (aControlId == ECtlOptionsBgImage)
	{
		TInt option = iBgImageChoiceList->CurrentItem();
		if ((option == EBgImageOptionSelect)  || ((option == EBgImageOptionUser) && (iZoomOptions.iBgImageFilename.Length() == 0)))
			SelectImageFileL();
	}
}

void CGeneralOptionsDialog::SelectImageFileL()
{
	_LIT(KMimeImage, "image/*");
	CDesCArray* mimeArray=new( ELeave ) CDesCArrayFlat( 2 );
	CleanupStack::PushL(mimeArray);
	mimeArray->AppendL(KMimeImage);

	TFileName	fileName;
	if (CQikSelectMediaFileDlg::RunDlgLD(*mimeArray, fileName))
	{
		iZoomOptions.iBgImageFilename = fileName;
		SetUserDefinedL();
		iBgImageChoiceList->SetCurrentItem(EBgImageOptionUser);
	}

	CleanupStack::PopAndDestroy();		// mimeArray

}


/************************************************
 *
 * SimpleTextInfoDialog
 *
 ************************************************/


CSimpleTextInfoDialog::CSimpleTextInfoDialog(TInt aTextIdOne, TInt aRichTextCtlIdOne, TInt aTextIdTwo, TInt aRichTextCtlIdTwo, TBool aSimpleTextResIdOneIsArray, TBool aSimpleTextResIdTwoIsArray)
	: iRichTextCtlIdOne(aRichTextCtlIdOne), iSimpleTextResIdOne(aTextIdOne),
	  iRichTextCtlIdTwo(aRichTextCtlIdTwo), iSimpleTextResIdTwo(aTextIdTwo),
	  iSimpleTextResIdOneIsArray(aSimpleTextResIdOneIsArray), iSimpleTextResIdTwoIsArray(aSimpleTextResIdTwoIsArray),
	  iSetDialogBackground(ETrue)
{
}

void CSimpleTextInfoDialog::PreLayoutDynInitL()
{
	CEikRichTextEditor*		richTextEditor;
	
	if (iRichTextCtlIdOne != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdOne));
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdOne, iSimpleTextResIdOne);
	}

	if (iRichTextCtlIdTwo != -1)
	{
		richTextEditor=STATIC_CAST(CEikRichTextEditor*, Control(iRichTextCtlIdTwo));
		richTextEditor->Border().SetType(ENone);
		PreLayoutDynInitRichTextL(*richTextEditor, iRichTextCtlIdTwo, iSimpleTextResIdTwo);
	}
}


void CSimpleTextInfoDialog::PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aSimpleTextResId)
{
	iEikonEnv->BusyMsgL(_L("Opening"));
	aRichTextEditor.SetEdwinSizeObserver(this);
	if (iSetDialogBackground)
		aRichTextEditor.SetBackgroundColorL(iEikonEnv->Color(EColorDialogBackground));
	aRichTextEditor.SetSize(aRichTextEditor.Size()); // Set the size of the edwin

		// no scrollbars for us
	aRichTextEditor.CreateScrollBarFrameL(); // Create the scrollbars
	aRichTextEditor.ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, iWantVertScrollbar ? CEikScrollBarFrame::EAuto: CEikScrollBarFrame::EOff);

	ShowTextL(*aRichTextEditor.RichText(), aRichTextCtlId, aSimpleTextResId);

	aRichTextEditor.UpdateAllFieldsL(); // Updates all the fields in the document

	aRichTextEditor.UpdateScrollBarsL();
}


void CSimpleTextInfoDialog::PostLayoutDynInitL()
{
	Layout();
	iEikonEnv->BusyMsgCancel();
}

TBool CSimpleTextInfoDialog::HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize)
{
	if ((aEventType == EEventSizeChanging))
		aEdwin->SetSize(aDesirableEdwinSize);
	return EFalse;
}

void CSimpleTextInfoDialog::ShowTextL(CRichText& aRichText, TInt /*aRichTextCtlId*/, TInt aResId)
{
	if (aResId != -1)
	{
		if ( ((aResId == iSimpleTextResIdOne) && (iSimpleTextResIdOneIsArray)) ||
		     ((aResId == iSimpleTextResIdTwo) && (iSimpleTextResIdTwoIsArray))
		   )
		{
			CDesCArrayFlat* desArray = CEikonEnv::Static()->ReadDesCArrayResourceL(aResId);
			CleanupStack::PushL(desArray);

			CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();

			TInt	count = desArray->Count();
			for (TInt i=0 ;i<count ; i++)
				parser->ParseL(desArray->operator[](i), aRichText);

			CleanupStack::PopAndDestroy(parser);
			CleanupStack::PopAndDestroy(desArray);
		}
		else
		{
			HBufC*		text =	CEikonEnv::Static()->AllocReadResourceLC(aResId);
			ShowSimpleTextL(*text, aRichText);
			CleanupStack::PopAndDestroy(text);
		}
	}
}

void CSimpleTextInfoDialog::ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText)
{
	CSimpleTextFormatParser* parser = CSimpleTextFormatParser::NewLC();
	parser->ParseL(aSimpleText, aRichText);

	CleanupStack::PopAndDestroy(parser);
}
