/*******************************************************************
 *
 *	File:		Zoom.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "Zoom.h"
#include "Dialogs.h"
#include <ezoom.rsg>
#include <eikenv.h>
#include <eikdialg.h>
#include <eikmenup.h>


CApaDocument* CZoomApplication::CreateDocumentL()
{
	CZoomDocument*	doc = new (ELeave) CZoomDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CZoomDocument::CZoomDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CZoomDocument::~CZoomDocument()
{
}

void CZoomDocument::ConstructL()
{
}


CEikAppUi* CZoomDocument::CreateAppUiL()
{
	return new(ELeave) CZoomAppUi;
}


CZoomAppUi::~CZoomAppUi()
{
	delete iAppView;

	delete iZoomEngine;
}


void CZoomAppUi::ConstructL()
{
	BaseConstructL();

	iZoomEngine = CZoomEngine::NewL(CEikonEnv::Static()->WsSession());

	iAppView=new(ELeave) CZoomAppView;
	iAppView->ConstructL(ClientRect());
}


void CZoomAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit: 
			Exit();
			break;

		case EEikCmdAbout:
			DisplayAboutDialogL();
			break;

		case EEikCmdOptions:
			DisplayOptionsDialogL();
			break;

		case EEikCmdZoomingStartStop:
		{
			iZoomEngine->StartStopZooming();
			break;
		}
		case EEikCmdHelp:
			DisplayHelpDialogL();
			break;

		default:
			break;
	}
}

void CZoomAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CEikDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}

void CZoomAppUi::DisplayOptionsDialogL()
{
	TZoomOptions	zoomOptions = iZoomEngine->Options();

	CEikDialog*	dialog = new(ELeave) CGeneralOptionsDialog(zoomOptions);
	if (dialog->ExecuteLD(R_DIALOG_OPTIONS) == EEikBidOk)
	{
		iZoomEngine->SaveOptions(zoomOptions);
	}
}

void CZoomAppUi::DisplayHelpDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EZOOM_HELP, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DIPLAY_EZOOM_HELP);
}


void CZoomAppUi::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
{
	switch (aMenuId)
	{
		case R_ZOOM_MENU:
			DynInitZoomMenuPaneL(aMenuPane);
			break;
		default:
			break;
	}
}


void CZoomAppUi::DynInitZoomMenuPaneL(CEikMenuPane* aMenuPane)
{
	_LIT(KMenuItemStart, "Start zooming");
	_LIT(KMenuItemStop, "Stop zooming");
	if (iZoomEngine->IsZooming())
		aMenuPane->SetItemTextL(EEikCmdZoomingStartStop, KMenuItemStop);
	else
		aMenuPane->SetItemTextL(EEikCmdZoomingStartStop, KMenuItemStart);
}

void CZoomAppUi::HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination)
{
	if (!iZoomEngine->ConsumeEvent(aEvent))
		CEikAppUi::HandleWsEventL(aEvent, aDestination);
	else
		NotificationReceived();
}

void CZoomAppUi::NotificationReceived()
{
	iZoomEngine->StartStopZooming();
}

void CZoomAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();
}


CZoomAppView::~CZoomAppView()
{
}


void CZoomAppView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	gc.Clear(aRect);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CZoomApplication;
}



TUid CZoomApplication::AppDllUid() const
{
	return KUidZoomApp;
}



