/*******************************************************************
 *
 *	File:		Mmu.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "zoom.h"
#include "mmu.h"

/*
 * Routines stolen from the Arlo project (http://www.yipton.net)
 * Also, details can be found in ARM 920T, technical reference manual (DDI0151B_920T_trm.pdf
 * on www.arm.com)
 *
 */

void Panic(TInt)
{
	// can't really PANIC, but this will bring the system down as well ;-)
	*((TInt*)0) = 0;
}

static inline TUint32 ArloExamine(TUint32 aVal)
{
	return *((TUint32*)aVal);
}


void RemapLinearAddress(TUint32 aLinearAdr, TUint32 aPhsyAdr, TUint32 aBit2And3)
{
	TUint		pageDirectoryIndex = (aLinearAdr >> 20);
	TUint		pageTableIndex = ((aLinearAdr & 0x000ff000) >> 12);

	TUint32 levelOneDescriptor = PAGE_DIRECTORY_LINEAR_ADDRESS + (pageDirectoryIndex << 2);
	levelOneDescriptor = ArloExamine(levelOneDescriptor);
	
	if (levelOneDescriptor == 0)
	{
		Panic(EZoomLddPanicLevelOneIsZero);
	} 
	else if ((levelOneDescriptor & 0x3) == 0x2) 
	{
		Panic(EZoomLddPanicLevelOneIsSectionDescriptor);
	}
	else if ((levelOneDescriptor & 0x3) != 0x1) 
	{
		Panic(EZoomLddPanicLevelOneIsFaultOrReserved);
	}


	TUint32	pageId = 0x41080000 + (pageDirectoryIndex << 2);
	pageId = ArloExamine(pageId);

	if (pageId == 0)
	{
		Panic(EZoomLddPanicPageIdIsZero);
	}


	TUint32		page = PAGE_TABLE_LINEAR_BASE + ((pageId >> 16) * 1024) + (pageTableIndex << 2);
	
		/*
		 * Finally map to phys address
		 */
	TUint32*	levelTwoDescriptor = (TUint32*) page;
	*levelTwoDescriptor = (aPhsyAdr & 0xfffff000) | (*levelTwoDescriptor & 0xff3) | aBit2And3;
}


TUint32 Linear2Physical(TUint32 aLinearAdr)
{
	TUint		pageDirectoryIndex = (aLinearAdr >> 20);
	TUint		pageTableIndex = ((aLinearAdr & 0x000ff000) >> 12);
	TUint		offsetInPage = aLinearAdr & 0xfff;

	TUint32 levelOneDescriptor = PAGE_DIRECTORY_LINEAR_ADDRESS + (pageDirectoryIndex << 2);
	levelOneDescriptor = ArloExamine(levelOneDescriptor);
	
	if (levelOneDescriptor == 0)
	{
		Panic(EZoomLddPanicLevelOneIsZero);
	} 
	else if ((levelOneDescriptor & 0x3) == 0x2) 
	{
		Panic(EZoomLddPanicLevelOneIsSectionDescriptor);
	}
	else if ((levelOneDescriptor & 0x3) != 0x1) 
	{
		Panic(EZoomLddPanicLevelOneIsFaultOrReserved);
	}

	TUint32	pageId = 0x41080000 + (pageDirectoryIndex << 2);
	pageId = ArloExamine(pageId);

	if (pageId == 0)
	{
		Panic(EZoomLddPanicPageIdIsZero);
	}


	TUint32		page = PAGE_TABLE_LINEAR_BASE + ((pageId >> 16) * 1024) + (pageTableIndex << 2);
	page = ArloExamine(page);
	TUint32		physAddr = (page & 0xfffff000) +  offsetInPage;

	return physAddr;
}

