/*******************************************************************
 *
 *	File:		Zoom.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "zoom.h"
#include "zoom_ldd_if.h"
#include "mmu.h"
#ifdef __MARM__
#include "kc_arm.h"
#endif

/***********************************************************
 *
 * Dummy implementation of kernel functionality in absense 
 * of EKERN.LIB
 *
 ***********************************************************/

DLogicalDevice::DLogicalDevice()
{
}

DLogicalDevice::~DLogicalDevice()
{
}


TInt DLogicalDevice::Remove()
{
	return KErrNone;
}

TBool DLogicalDevice::QueryVersionSupported(const TVersion& /*aVer*/) const
{
	return ETrue;
}

TBool DLogicalDevice::IsAvailable(TInt /*aUnit*/, const TDesC* /*aDriver*/, const TDesC* /*anInfo*/) const
{
	return ETrue;
}


DLogicalChannel::DLogicalChannel(DLogicalDevice * aDevice) : iDevice(aDevice)
{
}

DLogicalChannel::~DLogicalChannel()
{
	iDevice->iOpenChannels--;
}


void DLogicalChannel::Close()
{
	// not sure what void DLogicalChannel::Close() does, but I need to call CObject::Close();
	CObject::Close();
}

void DLogicalChannel::DoCreateL(TInt /*aUnit*/,CBase* /*aDriver*/, const TDesC* /*anInfo*/, const TVersion& /*aVer*/)
{
}

TInt DLogicalChannel::DoControl(TInt /*aFunction*/ ,TAny* /*a1*/, TAny* /*a2*/)
{
	return KErrNone;
}


/***********************************************************
 *
 * Actual driver code
 *
 ***********************************************************/


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}


class DLddZoom : public DLogicalDevice
{
public:
	~DLddZoom();
	DLddZoom();
	virtual TInt Install();
	virtual void GetCaps(TDes8&) const {}
	virtual DLogicalChannel* CreateL();
};


DLddZoom::~DLddZoom()
{
}


class DChannelZoom : public DLogicalChannel
{
public:
	DChannelZoom(DLogicalDevice * aDevice);
	~DChannelZoom();                  
protected:
	virtual void DoCancel(TInt){};
	virtual void DoRequest(TInt, TAny*, TAny*){};
	virtual TInt DoControl(TInt aFunction, TAny* a1, TAny* a2);
	virtual void DoCreateL(TInt /*aUnit*/, CBase* /*aPdd*/, const TDesC* /*anInfo*/, const TVersion& /*aVer*/){};
};



EXPORT_C DLogicalDevice *CreateLogicalDevice()
{
	return(new DLddZoom);
}


DLddZoom::DLddZoom()
{
	iUnitsMask = 1;
	iVersion = TVersion(1, 0 , 0);
}


TInt DLddZoom::Install()
{
	TPtrC lddName = LDD_NAME;
	return SetName(&lddName);
}


DLogicalChannel *DLddZoom::CreateL()
{
	return (new (ELeave) DChannelZoom(this));
}


DChannelZoom::DChannelZoom(DLogicalDevice * aDevice) : DLogicalChannel(aDevice)
{
}

DChannelZoom::~DChannelZoom()
{
}


static inline TUint32 ToLevel2DescriptorBits2And3(TBool aCache, TBool aBuffer)
{
	TUint32 bits =0;
	if (aCache)
		bits |= (1 << 3);
	if (aBuffer)
		bits |= (1 << 2);
	return bits;
}


TInt DChannelZoom::DoControl(TInt aFunction, TAny* a1, TAny* a2)
{
	TInt	err = KErrNone;
	switch (aFunction)
	{
		case ZOOM_LDD_CMD_LIN_TO_PHYS:
		{
			TPhysAddr*	physAddr = (TPhysAddr*)a2;
			*physAddr = Linear2Physical((TLinAddr)a1);
			break;
		}

		
		case ZOOM_LDD_CMD_REMAP_LIN_ADDR:
		{
			zoom_cmd_remap_lin_addr_t*		cmd = (zoom_cmd_remap_lin_addr_t*)a1;
			TUint32 bits = ToLevel2DescriptorBits2And3(cmd->iChache, cmd->iBuffer);
			RemapLinearAddress(cmd->iLinAddr, cmd->iPhysAddr, bits);
			break;
		}

		default:
		{
			err = KErrNotSupported;
			break;
		}
	}
	
	return err;
}


